
#include "Precomp.h"

//In this cpp file are declared the global functions

//----------------------------------------------------------------------------------------
//-----------------				Globals from Max				--------------------------
//----------------------------------------------------------------------------------------
ClassDesc2* GetSuperUtilityDesc() {return &SuperUtilityDesc;}

TCHAR *GetString(int id)
{
	static TCHAR buf[256];

	if (hInstance)
		return LoadString(hInstance, id, buf, sizeof(buf)) ? buf : NULL;
	return NULL;
}


//----------------------------------------------------------------------------------------
//-----------------					My globals					--------------------------
//----------------------------------------------------------------------------------------


//-----------------------------------------------------------------------------------------
//-----------------					Variables					---------------------------
//-----------------------------------------------------------------------------------------
Matrix3				identity(1);						//IdentityMatrix
ModContext			mc		(&identity, NULL, NULL);	// new MC for adding modifier
static const char*	Levels[]			= {"Object", "Vertex", "Edge", "Triangles", "Polygons", "Element"};
BOOL				IsOurDialogVisible	= FALSE;//To know if our dialog is visble
Interface*			ip					= NULL;	//Interface pointer needed in all plugins


//-----------------------------------------------------------------------------------------
//-----------------					Functions					---------------------------
//-----------------------------------------------------------------------------------------
//Get the level which is selected in our combo box (object, vertex, edge etc.)
int GetSubObjectLevelFromComboBox(HWND hWnd)
{
	int Level = 0;

	HWND HWndComboBox = GetDlgItem(hWnd,IDC_COMBO_SELLEVEL);
	if (HWndComboBox)
	{
		//Get the current selected item
		const int CurSel = 
		SendMessage( 
		  HWndComboBox,         // handle to destination window 
		  CB_GETCURSEL,         // message to send
		  (WPARAM) 0,	        // not used; must be zero
		  (LPARAM) 0		    // not used; must be zero
		);
		if (CurSel == CB_ERR)	//Error
			return 0;

		//Now get the data stored in the selected item (it is the subobjectlevel taht we need)
		Level = 
		SendMessage( 
		HWndComboBox,           // handle to destination window 
	  CB_GETITEMDATA,           // message to send
	  (WPARAM) CurSel,          // item index
	  (LPARAM) 0	            // not used; must be zero
	  );

	  if (Level == CB_ERR)	//Error
		return 0;
	}
	
	//Now I remember why I do like MFC..... :-)

	return Level;
}

//Fill the combo box with our subobject levels : object, vertex, edge, triangle, polygon, element
void FillTheComboBox(HWND hWnd)
{
	HWND HWndComboBox = GetDlgItem(hWnd,IDC_COMBO_SELLEVEL);
	if (HWndComboBox)
	{
		//reset the content of the combo box :
		SendMessage( 
		  HWndComboBox,			  // handle to destination window 
		  CB_RESETCONTENT,        // message to send
		  (WPARAM) 0,		      // not used; must be zero
		  (LPARAM) 0	          // not used; must be zero
		);
		
		//Now AddThe strings we need
		for (int i=0;i<6;i++)
		{
			//Add our predefined string
			SendMessage( 
			  HWndComboBox,       // handle to destination window 
			  CB_ADDSTRING,       // message to send
			  (WPARAM) 0,         // not used; must be zero
			  (LPARAM) Levels[i]  // string to add (LPCTSTR)
			);

			//Add the index (it is the subobject )level as a data of the item
			SendMessage( 
			  HWndComboBox,       // handle to destination window 
			  CB_SETITEMDATA,     // message to send
			  (WPARAM) i,		  // item index
			  (LPARAM) i          // item data (DWORD)
			);

		}

		//Select the 1st item 
		SendMessage( 
		  HWndComboBox,           // handle to destination window 
		  CB_SETCURSEL,           // message to send
		  (WPARAM) 0,	          // item index
		  (LPARAM) 0			  // not used; must be zero
		);
	}
	
	//Now I remember why I do like MFC..... :-)
}

//This function adds the edit mesh modifier on the selected objects and changes the subobject level to "Level"
// Which is Object, Vertex, Edge etc. The pointer on the edit mesh is then returned
//if the edit mesh is already on the top of the modifier stack of all selected objects, it just returns its pointer
//It will be useless to add another edit mesh if one is already on the top of the stack

//The original function that I have modified was given to me by Harry Denholm. Thanx to him !

Modifier* PutEditMeshOnTopOfStack(int Level)
{
	Modifier *EM = NULL;
	Modifier *sm = NULL;

	BOOL NeedToUpdateStack = FALSE;
	
	// new MC for adding modifier
	int nbSelNode = ip->GetSelNodeCount();
	if (nbSelNode >0)
	{
		// Get the first selected node to work on it
		INode *node = ip->GetSelNode(0);

		//Get its derived object if it exists
		Object *pObj = node->GetObjectRef();
		IDerivedObject *pDerObj = NULL;

		// check for presence of derived object
		// if none exists, create a new one
		if( pObj->SuperClassID() == GEN_DERIVOB_CLASS_ID) 
		{
			pDerObj = static_cast<IDerivedObject *> ( pObj );
		}
		else
		{
			pDerObj = CreateDerivedObject();
			pDerObj->TransferReferences(pObj);
			pDerObj->ReferenceObject(pObj);
		}

		// Now check if the editmesh modifier on the top of the modifier stack
		// if it isn't there, add a new instance
		Modifier *tMod = NULL;
		if(pDerObj->NumModifiers()>0)
		{
			// Get top of stack modifier
			tMod = pDerObj->GetModifier(0);

			// Is it an edit mesh ?
			if( tMod->ClassID() == Class_ID(EDITMESH_CLASS_ID,0) )
			{
				// yep, just store the pointer
				EM = tMod;
			}
			else
				// no, nuke the sm ptr for posterity
				EM = NULL;
		}

		// hold animation
		SuspendAnimate();
		AnimateOff();

		// EM is still NULL - So we have to add a new modifier
		if(EM==NULL)
		{
			NeedToUpdateStack = TRUE;	//We'll need to update the stack
			
			// turn off undo stack
			BOOL was_hold = theHold.Holding();
			if (was_hold)
				theHold.Suspend();

			// Instance new EditMesh modifier
			EM = static_cast<Modifier*> ( ip->CreateInstance(OSM_CLASS_ID,	Class_ID(EDITMESH_CLASS_ID,0)));

			// Add mod to derived object container
			pDerObj->AddModifier(EM,&mc,0);

			// resume undo stack
			if (was_hold)
				theHold.Resume();
			
			// we got good mod ptr?
			assert(EM);

			// update derived object
			pDerObj->NotifyDependents(FOREVER,PART_ALL,REFMSG_CHANGE);
			// make damn sure everything knows something has changed
			EM->NotifyDependents(FOREVER,PART_ALL,REFMSG_CHANGE);
			node->NotifyDependents(FOREVER,PART_ALL,REFMSG_CHANGE);
			node->NotifyDependents(FOREVER,0,REFMSG_SUBANIM_STRUCTURE_CHANGED);
		}

		// restore animation
		ResumeAnimate();
	}

	//We now have in EM the editMeshModifier pointer in the first selected node
	//so check the others selected objects
	for (int i=1;i<nbSelNode;i++)
	{
		// Get the selected node to work on it
		INode *node = ip->GetSelNode(i);

		Object *pObj = node->GetObjectRef();
		IDerivedObject *pDerObj = NULL;

		// check for presence of derived object
		// if none exists, create a new one
		if( pObj->SuperClassID() == GEN_DERIVOB_CLASS_ID) 
		{
			pDerObj = static_cast<IDerivedObject *> ( pObj );
		}
		else
		{
			pDerObj = CreateDerivedObject();
			pDerObj->TransferReferences(pObj);
			pDerObj->ReferenceObject(pObj);
		}

		// find if the same EM modifier is on top of stack
		// if it isn't there, add our Edit Mesh
		Modifier *tMod = NULL;

		BOOL NeedToAddEM = TRUE;

		if(pDerObj->NumModifiers()>0)
		{
			// Get top of stack modifier
			tMod = pDerObj->GetModifier(0);

			
			// Is it already on the top?
			if( tMod->ClassID() == Class_ID(EDITMESH_CLASS_ID,0) )
			{
				// yep, just store the pointer
				if (EM = tMod)
					NeedToAddEM = FALSE; //It is already on top of the stack
			}
		}

		if (NeedToAddEM)
		{
			NeedToUpdateStack = TRUE;

			// hold animation
			SuspendAnimate();
			AnimateOff();
		
			// turn off undo stack
			BOOL was_hold = theHold.Holding();
			if (was_hold)
				theHold.Suspend();

				// Add mod to derived object container
				pDerObj->AddModifier(EM,&mc,0);


			// resume undo stack
			if (was_hold)
				theHold.Resume();

			// restore animation
			ResumeAnimate();

			// update derived object
			pDerObj->NotifyDependents(FOREVER,PART_ALL,REFMSG_CHANGE);
			// make damn sure everything knows something has changed
			EM->NotifyDependents(FOREVER,PART_ALL,REFMSG_CHANGE);
			node->NotifyDependents(FOREVER,PART_ALL,REFMSG_CHANGE);
			node->NotifyDependents(FOREVER,0,REFMSG_SUBANIM_STRUCTURE_CHANGED);
		}
	}
	
	int b = ip->GetCommandPanelTaskMode();	//Check if we are in modify mode

	if (NeedToUpdateStack)
	{
		// jump to modifier panel
		ip->ExecuteMAXCommand(MAXCOM_MODIFY_MODE);
	}
	else
	{	//If we are not in modify mode, set the modify mode active
		if (b != TASK_MODE_MODIFY)
			ip->SetCommandPanelTaskMode(TASK_MODE_MODIFY);	
	}

	//Set the select mode active
	ip->ExecuteMAXCommand(MAXCOM_SELECT);

	//Now we can set the subobject level	
	if (ip->GetNumSubObjectLevels() >= Level)
		ip->SetSubObjectLevel(Level); 
	
	// Redraw the viewports
	ip->ForceCompleteRedraw();

	//Return the edit mesh pointer
	return EM;
}

//Collapse the modifier stack of all selected nodes
void Collapse()
{
	//Memorize the subobject level
	const int Subobjectlevel = ip->GetSubObjectLevel();

	//Set the subobjectlevel in object selection mode
	ip->SetSubObjectLevel(0);

	INode	  *node	= NULL;
	TriObject *tobj = NULL;

	//For all selected nodes
	for (int i=0; i<ip->GetSelNodeCount(); i++) 
	{
		// Get a selected node
		node = ip->GetSelNode(i);

		if (!node)
			continue;

		//Collapse it !
		// Eval the node's object (exclude WSMs)
		Object *oldObj	= node->GetObjectRef();
		ObjectState os	= oldObj->Eval(ip->GetTime());
		Object *obj		= static_cast<Object*> (os.obj->Clone() );

		if (os.obj->CanConvertToType(triObjectClassID)) 
		{
			// Convert it to a TriObject and make that the new object
			tobj = static_cast<TriObject*> (obj->ConvertToType(ip->GetTime(),triObjectClassID));
			oldObj->SetAFlag(A_LOCK_TARGET);
			node->SetObjectRef(tobj);		
			oldObj->ClearAFlag(A_LOCK_TARGET);
			if (obj!=tobj) 
				obj->AutoDelete();

			//Update
			node->NotifyDependents(FOREVER,PART_ALL,REFMSG_CHANGE);
			node->NotifyDependents(FOREVER,0,REFMSG_SUBANIM_STRUCTURE_CHANGED);
		}
	}

	// jump to modifier panel to update the modifications
	ip->ExecuteMAXCommand(MAXCOM_MODIFY_MODE);

	//Reset the subobjectlevel memorized (if posible)
	if (ip->GetNumSubObjectLevels() >= Subobjectlevel)
		ip->SetSubObjectLevel(Subobjectlevel);

	//Redraw
	ip->ForceCompleteRedraw();
	GetSystemSetting(SYSSET_CLEAR_UNDO);	//flush the undo buffer
}
